@@ -4,7 +4,7 @@ use warnings qw(FATAL all NONFATAL misc);
 use 5.010; no if $] >= 5.017011, warnings => "experimental";
 
 use Carp qw(carp croak confess);
-our $VERSION = '0.101_102';
+our $VERSION = '0.101_103';
 use mro 'c3';
 
 use Scalar::Util qw/weaken/;
@@ -194,11 +194,12 @@ sub handle {
   }
 
   my $sub = $YATT->find_handler($ext, $file, $CON);
-  $sub->($YATT, $CON, $file);
+
+  my $result = $sub->($YATT, $CON, $file);
 
   try_invoke($CON, 'flush_headers');
 
-  $CON;
+  $result;
 }
 
 sub render {
